// Fill out your copyright notice in the Description page of Project Settings.


#include "AIEnemy_Controller.h"

#include "AIEnemy_Actor.h"
#include "AIEnemy_Character.h"
#include "BehaviorTree/BehaviorTree.h"
#include "BehaviorTree/BehaviorTreeComponent.h"
#include "BehaviorTree/BlackboardComponent.h"
#include "FiveStackPotionGame/FiveStackPotionGameCharacter.h"
#include "Perception/AIPerceptionComponent.h"
#include "Kismet\KismetSystemLibrary.h"
#include "Math/UnitConversion.h"

AAIEnemy_Controller::AAIEnemy_Controller()
{
	BlackboardComponent = CreateDefaultSubobject<UBlackboardComponent>(TEXT("Blackboard Component"));
		check(BlackboardComponent);

	BehaviorTreeComponent = CreateDefaultSubobject<UBehaviorTreeComponent>(TEXT("Behaviour Tree Component"));
		check(BehaviorTreeComponent);

	//Setup the perception component
	PerceptionComp = CreateDefaultSubobject<UAIPerceptionComponent>(TEXT("AIPerception Component"));
	SenseConfig_Sight = CreateDefaultSubobject<UAISenseConfig_Sight>(TEXT("Sight Config"));
	PerceptionComp->ConfigureSense(*SenseConfig_Sight);
	PerceptionComp->SetDominantSense(SenseConfig_Sight->GetSenseImplementation());
	PerceptionComp->OnPerceptionUpdated.AddDynamic(this, &AAIEnemy_Controller::OnPawnDetect);

	//Allow sight to see everything
	SenseConfig_Sight->DetectionByAffiliation.bDetectEnemies = true;
	SenseConfig_Sight->DetectionByAffiliation.bDetectFriendlies = true;
	SenseConfig_Sight->DetectionByAffiliation.bDetectNeutrals = true;

}

void AAIEnemy_Controller::OnPossess(APawn* InPawn)
{
	Super::OnPossess(InPawn);
	if(InPawn == nullptr)
	{
		return;
	}
	Enemy = Cast<AAIEnemy_Character>(InPawn);
	if(Enemy)
	{
		if(Enemy->BehaviorTree)
		{
			BlackboardComponent->InitializeBlackboard(*(Enemy->BehaviorTree->BlackboardAsset));
			BehaviorTreeComponent->StartTree(*(Enemy->BehaviorTree));
		}
	}
}

UBlackboardComponent* AAIEnemy_Controller::GetBlackboard()
{
	return BlackboardComponent;
}

void AAIEnemy_Controller::OnPawnDetect(const TArray<AActor*>&DetectedPawn)
{
	//Check to see if this enemy has spotted the player
	for(AActor* Actor : DetectedPawn)
	{
		if(Actor->IsA<AFiveStackPotionGameCharacter>()) 
		{
			UE_LOG(LogTemp, Warning, TEXT("CAN SEE PLAYER"));
			Enemy->bscreaming = true;
			
			AlertEnemies(Actor);
			BlackboardComponent->SetValueAsObject("Player", Actor);
			return;
		}
	}

	//If the enemy can no longer see the actor
	// Make sure to delete any previous reference from the blackboard
	BlackboardComponent->SetValueAsObject("Player", nullptr);
}

void AAIEnemy_Controller::AlertEnemies(AActor* PlayerActor)
{
	// Set what actors to seek out from it's collision channel
	TArray<TEnumAsByte<EObjectTypeQuery>> traceObjectTypes;
	traceObjectTypes.Add(UEngineTypes::ConvertToObjectType(ECollisionChannel::ECC_Pawn));

	//Ignore any specific actors
	TArray<AActor*> ignoreActors;
	//Ignore self
	ignoreActors.Init(this, 1);

	//Array of actors that inside the radius of the sphere
	TArray<AActor*> outActors;

	//Radius of Sphere
	float radius = 500.0f;
	//Position of spheres spawn locaiton
	FVector sphereSpawnLoc = GetPawn()->GetActorLocation();
	//Class that the sphere should hit against
	UClass* enemiesInRadius = AAIEnemy_Character::StaticClass();

	//Generate radius around this enemy
	UKismetSystemLibrary::SphereOverlapActors(GetWorld(), sphereSpawnLoc, radius, traceObjectTypes, enemiesInRadius, ignoreActors, outActors);

	for (AActor* overlappedActor : outActors) 
	{
		UE_LOG(LogTemp, Log, TEXT("OverlappedActor: %s"), *overlappedActor->GetName());

		AAIEnemy_Character* AICharacter = Cast<AAIEnemy_Character>(overlappedActor);
		if (AICharacter) 
		{
			UE_LOG(LogTemp, Warning, TEXT("FOUND FELLOW MATE"));
			AICharacter->chasePlayer(PlayerActor);
		}
	}
}

void AAIEnemy_Controller::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);

}




